@@ -229,7 +229,7 @@ namespace System.Web.UI.WebControls {
 				if (dataView == null) {
 					SqlDataSourceSelectingEventArgs selectingArgs = new SqlDataSourceSelectingEventArgs (command, arguments);
 					OnSelecting (selectingArgs);
-					if (selectingArgs.Cancel) {
+					if (selectingArgs.Cancel || !PrepareNullParameters (command, CancelSelectOnNullParameter)) {
 						return null;
 					}
 					try {
@@ -279,7 +279,7 @@ namespace System.Web.UI.WebControls {
 			else {
 				SqlDataSourceSelectingEventArgs selectingArgs = new SqlDataSourceSelectingEventArgs (command, arguments);
 				OnSelecting (selectingArgs);
-				if (selectingArgs.Cancel) {
+				if (selectingArgs.Cancel || !PrepareNullParameters (command, CancelSelectOnNullParameter)) {
 					return null;
 				}
 
@@ -304,6 +304,20 @@ namespace System.Web.UI.WebControls {
 			}
 		}
 
+		static bool PrepareNullParameters (DbCommand command, bool cancelIfHas)
+		{
+			for (int i = 0; i < command.Parameters.Count; i++) {
+				DbParameter param = command.Parameters [i];
+				if (param.Value == null && ((param.Direction & ParameterDirection.Input) != 0)) {
+					if (cancelIfHas)
+						return false;
+					else
+						param.Value = DBNull.Value;
+				}
+			}
+			return true;
+		}
+
 		public int Update (IDictionary keys, IDictionary values, IDictionary oldValues)
 		{
 			return ExecuteUpdate (keys, values, oldValues);
@@ -441,19 +455,16 @@ namespace System.Web.UI.WebControls {
 			}
 		}
 
-		private DbParameter CreateDbParameter (string name, object value)
+		DbParameter CreateDbParameter (string name, object value)
 		{
 			return CreateDbParameter (name, value, ParameterDirection.Input, -1);
 		}
 		
-		private DbParameter CreateDbParameter (string name, object value, ParameterDirection dir, int size)
+		DbParameter CreateDbParameter (string name, object value, ParameterDirection dir, int size)
 		{
 			DbParameter dbp = factory.CreateParameter ();
 			dbp.ParameterName = ParameterPrefix + name;
-			if (value == null)
-				dbp.Value = DBNull.Value;
-			else
-				dbp.Value = value;
+			dbp.Value = value;
 			dbp.Direction = dir;
 			if (size != -1)
 				dbp.Size = size;
@@ -475,6 +486,23 @@ namespace System.Web.UI.WebControls {
 		{
 			TrackViewState ();
 		}
+
+		NotSupportedException CreateNotSupportedException (string capabilityName)
+		{
+			return new NotSupportedException ("Data source does not have the '" + capabilityName + "' capability enabled.");
+		}
+		
+		protected internal override void RaiseUnsupportedCapabilityError (DataSourceCapabilities capability)
+		{
+			if ((capability & DataSourceCapabilities.Sort) != 0 && !CanSort)
+				throw CreateNotSupportedException ("Sort");
+
+			if ((capability & DataSourceCapabilities.Page) != 0 && !CanPage)
+				throw CreateNotSupportedException ("Page");
+
+			if ((capability & DataSourceCapabilities.RetrieveTotalRowCount) != 0 && !CanRetrieveTotalRowCount)
+				throw CreateNotSupportedException ("RetrieveTotalRowCount");
+		}
 		
 		protected virtual void LoadViewState (object savedState)
 		{
@@ -517,10 +545,15 @@ namespace System.Web.UI.WebControls {
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private bool cancelSelectOnNullParameter = true;
+		bool cancelSelectOnNullParameter = true;
 		public bool CancelSelectOnNullParameter {
 			get { return cancelSelectOnNullParameter; }
-			set { cancelSelectOnNullParameter = value; }
+			set {
+				if (CancelSelectOnNullParameter == value)
+					return;
+				cancelSelectOnNullParameter = value;
+				OnDataSourceViewChanged (EventArgs.Empty);
+			}
 		}
 
 		public override bool CanDelete {
@@ -556,21 +589,26 @@ namespace System.Web.UI.WebControls {
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private ConflictOptions conflictDetection = ConflictOptions.OverwriteChanges;
+		ConflictOptions conflictDetection = ConflictOptions.OverwriteChanges;
 		public ConflictOptions ConflictDetection {
 			get { return conflictDetection; }
-			set { conflictDetection = value; }
+			set {
+				if (ConflictDetection == value)
+					return;
+				conflictDetection = value;
+				OnDataSourceViewChanged (EventArgs.Empty);
+			}
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private string deleteCommand = "";
+		string deleteCommand = String.Empty;
 		public string DeleteCommand {
 			get { return deleteCommand; }
 			set { deleteCommand = value; }
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private SqlDataSourceCommandType deleteCommandType = SqlDataSourceCommandType.Text;
+		SqlDataSourceCommandType deleteCommandType = SqlDataSourceCommandType.Text;
 		public SqlDataSourceCommandType DeleteCommandType {
 			get { return deleteCommandType; }
 			set { deleteCommandType = value; }
@@ -584,10 +622,15 @@ namespace System.Web.UI.WebControls {
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private string filterExpression = "";
+		string filterExpression;
 		public string FilterExpression {
-			get { return filterExpression; }
-			set { filterExpression = value; }
+			get { return filterExpression ?? string.Empty; }
+			set {
+				if (FilterExpression == value)
+					return;
+				filterExpression = value;
+				OnDataSourceViewChanged (EventArgs.Empty);
+			}
 		}
 
 		[EditorAttribute ("System.Web.UI.Design.WebControls.ParameterCollectionEditor, " + Consts.AssemblySystem_Design, "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
@@ -598,14 +641,14 @@ namespace System.Web.UI.WebControls {
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private string insertCommand = "";
+		string insertCommand = String.Empty;
 		public string InsertCommand {
 			get { return insertCommand; }
 			set { insertCommand = value; }
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private SqlDataSourceCommandType insertCommandType = SqlDataSourceCommandType.Text;
+		SqlDataSourceCommandType insertCommandType = SqlDataSourceCommandType.Text;
 		public SqlDataSourceCommandType InsertCommandType {
 			get { return insertCommandType; }
 			set { insertCommandType = value; }
@@ -623,22 +666,32 @@ namespace System.Web.UI.WebControls {
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private string oldValuesParameterFormatString = "{0}";
+		string oldValuesParameterFormatString = "{0}";
 		[DefaultValue ("{0}")]
 		public string OldValuesParameterFormatString {
 			get { return oldValuesParameterFormatString; }
-			set { oldValuesParameterFormatString = value; }
+			set {
+				if (OldValuesParameterFormatString == value)
+					return;
+				oldValuesParameterFormatString = value;
+				OnDataSourceViewChanged (EventArgs.Empty);
+			}
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private string selectCommand = "";
+		string selectCommand;
 		public string SelectCommand {
-			get { return selectCommand; }
-			set { selectCommand = value; }
+			get { return selectCommand != null ? selectCommand : string.Empty; }
+			set {
+				if (SelectCommand == value)
+					return;
+				selectCommand = value;
+				OnDataSourceViewChanged (EventArgs.Empty);
+			}
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private SqlDataSourceCommandType selectCommandType = SqlDataSourceCommandType.Text;
+		SqlDataSourceCommandType selectCommandType = SqlDataSourceCommandType.Text;
 		public SqlDataSourceCommandType SelectCommandType {
 			get { return selectCommandType; }
 			set { selectCommandType = value; }
@@ -649,21 +702,26 @@ namespace System.Web.UI.WebControls {
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private string sortParameterName = "";
+		string sortParameterName = String.Empty;
 		public string SortParameterName {
 			get { return sortParameterName; }
-			set { sortParameterName = value; }
+			set {
+				if (SortParameterName == value)
+					return;
+				sortParameterName = value;
+				OnDataSourceViewChanged (EventArgs.Empty);
+			}
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private string updateCommand = "";
+		string updateCommand = String.Empty;
 		public string UpdateCommand {
 			get { return updateCommand; }
 			set { updateCommand = value; }
 		}
 
 		// LAME SPEC: MSDN says value should be saved in ViewState but tests show otherwise.
-		private SqlDataSourceCommandType updateCommandType = SqlDataSourceCommandType.Text;
+		SqlDataSourceCommandType updateCommandType = SqlDataSourceCommandType.Text;
 		public SqlDataSourceCommandType UpdateCommandType {
 			get { return updateCommandType; }
 			set { updateCommandType = value; }
