@@ -12,7 +12,9 @@ using NUnit.Framework;
 using System;
 using System.IO;
 using System.Reflection;
+#if !MOBILE
 using System.Reflection.Emit;
+#endif
 using System.Runtime.InteropServices;
 using System.Security;
 using System.Text;
@@ -188,7 +190,7 @@ namespace MonoTests.System.Runtime.InteropServices
 				Assert.AreEqual ("fieldName", ex.ParamName, "#5");
 			}
 		}
-
+#if !MOBILE
 		[Test]
 		public void GetHINSTANCE ()
 		{
@@ -243,7 +245,7 @@ namespace MonoTests.System.Runtime.InteropServices
 				Assert.AreEqual ("m", ex.ParamName, "#5");
 			}
 		}
-
+#endif
 		[Test] // bug #319009
 		public void StringToHGlobalUni ()
 		{
@@ -256,6 +258,61 @@ namespace MonoTests.System.Runtime.InteropServices
 			Assert.AreEqual (19, s.Length, "#2");
 		}
 
+		[Test]
+		public void ReadIntByte ()
+		{
+			IntPtr ptr = Marshal.AllocHGlobal (4);
+			try {
+				Marshal.WriteByte (ptr, 0, 0x1);
+				Marshal.WriteByte (ptr, 1, 0x2);
+				Assert.AreEqual (0x1, Marshal.ReadByte (ptr));
+				Assert.AreEqual (0x1, Marshal.ReadByte (ptr, 0));
+				Assert.AreEqual (0x2, Marshal.ReadByte (ptr, 1));
+			} finally {
+				Marshal.FreeHGlobal (ptr);
+			}
+		}
+
+		[Test]
+		public void ReadInt16 ()
+		{
+			IntPtr ptr = Marshal.AllocHGlobal (64);
+			try {
+				Marshal.WriteInt16 (ptr, 0, 0x1234);
+				Marshal.WriteInt16 (ptr, 2, 0x4567);
+				Marshal.WriteInt16 (ptr, 5, 0x4567);
+				Assert.AreEqual (0x1234, Marshal.ReadInt16 (ptr));
+				Assert.AreEqual (0x1234, Marshal.ReadInt16 (ptr, 0));
+				Assert.AreEqual (0x4567, Marshal.ReadInt16 (ptr, 2));
+#if NET_4_5
+				Assert.AreEqual (0x4567, Marshal.ReadInt16 ((ptr + 5)));
+#endif
+				Assert.AreEqual (0x4567, Marshal.ReadInt16 (ptr, 5));
+			} finally {
+				Marshal.FreeHGlobal (ptr);
+			}
+		}
+
+		[Test]
+		public void ReadInt32 ()
+		{
+			IntPtr ptr = Marshal.AllocHGlobal (64);
+			try {
+				Marshal.WriteInt32 (ptr, 0, 0x12345678);
+				Marshal.WriteInt32 (ptr, 4, 0x77654321);
+				Marshal.WriteInt32 (ptr, 10, 0x77654321);
+				Assert.AreEqual (0x12345678, Marshal.ReadInt32 (ptr));
+				Assert.AreEqual (0x12345678, Marshal.ReadInt32 (ptr, 0));
+				Assert.AreEqual (0x77654321, Marshal.ReadInt32 (ptr, 4));
+#if NET_4_5
+				Assert.AreEqual (0x77654321, Marshal.ReadInt32 ((ptr + 10)));
+#endif
+				Assert.AreEqual (0x77654321, Marshal.ReadInt32 (ptr, 10));
+			} finally {
+				Marshal.FreeHGlobal (ptr);
+			}
+		}
+
 		[Test]
 		public void ReadInt32_Endian ()
 		{
@@ -277,6 +334,22 @@ namespace MonoTests.System.Runtime.InteropServices
 		}
 
 		[Test]
+		public void ReadInt64 ()
+		{
+			IntPtr ptr = Marshal.AllocHGlobal (16);
+			try {
+				Marshal.WriteInt64 (ptr, 0, 0x12345678ABCDEFL);
+				Marshal.WriteInt64 (ptr, 8, 0x87654321ABCDEFL);
+				Assert.AreEqual (0x12345678ABCDEFL, Marshal.ReadInt64 (ptr));
+				Assert.AreEqual (0x12345678ABCDEFL, Marshal.ReadInt64 (ptr, 0));
+				Assert.AreEqual (0x87654321ABCDEFL, Marshal.ReadInt64 (ptr, 8));
+			} finally {
+				Marshal.FreeHGlobal (ptr);
+			}
+		}
+
+		[Test]
+		[Category ("MobileNotWorking")]
 		public void BSTR_Roundtrip ()
 		{
 			string s = "mono";
@@ -286,6 +359,7 @@ namespace MonoTests.System.Runtime.InteropServices
 		}
 
 		[Test]
+		[Category ("MobileNotWorking")]
 		public void StringToBSTRWithNullValues ()
 		{
 			int size = 128;
@@ -585,6 +659,7 @@ namespace MonoTests.System.Runtime.InteropServices
 		}
 #endif
 
+#if !NET_2_1
 		[Test]
 		public void TestGetComSlotForMethodInfo ()
 		{
@@ -607,11 +682,7 @@ namespace MonoTests.System.Runtime.InteropServices
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
 				Assert.IsNotNull (ex.Message, "#4");
-#if NET_2_0
 				Assert.AreEqual ("m", ex.ParamName, "#5");
-#else
-				Assert.IsNull (ex.ParamName, "#5");
-#endif
 			}
 		}
 
@@ -630,7 +701,7 @@ namespace MonoTests.System.Runtime.InteropServices
 				Assert.AreEqual ("m", ex.ParamName, "#5");
 			}
 		}
-
+#endif
 		[Test]
 		public void TestPtrToStringAuto ()
 		{
@@ -659,7 +730,19 @@ namespace MonoTests.System.Runtime.InteropServices
 				Marshal.FreeHGlobal (ptr);
 			}
 		}
-
+#if !MOBILE
+		[Test]
+		public void TestGenerateProgIdForType()
+		{
+			string output;
+			
+			output = Marshal.GenerateProgIdForType(typeof(TestCoClass));
+			Assert.AreEqual ("MonoTests.System.Runtime.InteropServices.TestCoClass", output, "#1");
+			
+			output = Marshal.GenerateProgIdForType(typeof(TestCoClassWithProgId));
+			Assert.AreEqual ("CoClassWithProgId", output, "#2");
+		}
+#endif
 		[Test]
 		public void TestGlobalAlloc ()
 		{
@@ -667,14 +750,48 @@ namespace MonoTests.System.Runtime.InteropServices
 			mem = Marshal.ReAllocHGlobal (mem, (IntPtr) 1000000);
 			Marshal.FreeHGlobal (mem);
 		}
+		
+		[Test]
+		public void FreeHGlobal ()
+		{
+			// clear user doubts on assistly #6749
+			for (int i = 0; i < 1024; i++) {
+				IntPtr p = Marshal.AllocHGlobal (1024 * 1024);
+				Assert.AreNotEqual (IntPtr.Zero, p, i.ToString ());
+				Marshal.FreeHGlobal (p);
+			}
+		}
 
+		[StructLayout (LayoutKind.Sequential)]
+		public struct SimpleStruct2 {
+			public int a;
+			public int b;
+		}
+
+		[Test]
+		public void PtrToStructureNull ()
+		{
+			Assert.IsNull (Marshal.PtrToStructure (IntPtr.Zero, typeof (SimpleStruct2)));
+		}
+		
+#if NET_2_0
+		[Test]
+		public void TestGetExceptionForHR ()
+		{
+			const int E_OUTOFMEMORY = unchecked ((int) 0x8007000E);
+			const int E_INVALIDARG = unchecked ((int) 0X80070057);
+			
+			Exception ex = Marshal.GetExceptionForHR (E_OUTOFMEMORY);
+			Assert.AreEqual (typeof (OutOfMemoryException), ex.GetType (), "E_OUTOFMEMORY");
+			
+			ex = Marshal.GetExceptionForHR (E_INVALIDARG);
+			Assert.AreEqual (typeof (ArgumentException), ex.GetType (), "E_INVALIDARG");
+		}
+#endif
 		bool RunningOnUnix {
 			get {
-#if NET_2_0
-				return Environment.OSVersion.Platform == PlatformID.Unix;
-#else
-				return (int) Environment.OSVersion.Platform == 128;
-#endif
+				int p = (int) Environment.OSVersion.Platform;
+				return ((p == 4) || (p == 128) || (p == 6));
 			}
 		}
 
@@ -690,7 +807,7 @@ namespace MonoTests.System.Runtime.InteropServices
 			int nSize
 		);
 	}
-
+#if !NET_2_1
 	[ComImport()]
 	[Guid("AAAAAAAA-AAAA-AAAA-AAAA-AAAAAAAAAAAA")]
 	interface ITestDefault
@@ -744,5 +861,14 @@ namespace MonoTests.System.Runtime.InteropServices
 		{
 		}
 	}
+
+	[ProgId("CoClassWithProgId")]
+	public class TestCoClassWithProgId : ITestDispatch
+	{
+		public void DoNothing ()
+		{
+		}
+	}
+#endif
 }
 #endif
